##############################
##2013 Prefresher
##Day 1 - R Basics
##Stephen Pettigrew
##Harvard University
##pettigrew@fas.harvard.edu
##############################

## This file walks you through the most basic R stuff


########################################################
## PREAMBLE: Script and Console
########################################################

## R works by 
## 1. typing code in the console (next to the 'greater than' arrow) 
## 2. OR by typing on a script (like this one) and sending the lines of
## code to the console.

## Unless you are working on very simple calculations,
## you should always save your code as a script using a ".R" file extension.

## Every line that begins with '#' is ignored by the console.
## So you can add comments to your code without generating syntax errors.
## It's good practice to comment everything in your code.
## Somebody who knows R should be able to take your code and understand
## exactly what it's doing, without you there to explain to them.


#########################
##R can be used as a calculator
##########################

## Note: to send a line of code to your console, use either 
## (PC users:) "control + enter"
## (Mac users:) "command + return"

2+2

6*12

2^8

86/4

log(12) # the log function in R is based e, or the "natural log"

exp(2) ## equivalent to 2.71828^2 or e squared

2 * (6 + 2)^2 ##R preserves order of operations
2 * (8)^2 # equivalent to the expression above
2 * 64

2/9^2 ## another order of operations example
2/9^2 == 2/81 ## this statement is true
2/9^2 == (2/9) ^2 ## this one is false because you exponentiate before dividing






########################################################
## getting help in R
########################################################

## Let's say you want to get the mean of a vector...
## How could we do that?

## Let's say the vector is (86, 14, 9)

# we could calculate the mean by brute force
(86 + 14 + 9) / 3


## But what if the vector has 100 or 1,000,000 entries?
## Maybe there's a mean function in R
## Let's see...

## put two question marks in front of a word to look it up in the R help files
??mean

## if we already know the name of the function we want to learn about,
## we can look it up using a single question mark
?mean

## so now let's try to take the mean of our vector
mean(86, 14, 9) ## that gives us the wrong answer, so the code is wrong!

## We must have used the wrong syntax.
## The "Arguments" section of the mean help file tells us that
## x has to be an R object that's a "numeric/logical vector."
## But what's that actually mean?





## R is an object-oriented programming language.
## Objects can be vectors, matrices, functions, data, constants, lists, etc.
## Store an object for later retrieval by using " <- " as assignment operator.

## Objects are stored in what we call the "workspace."
## If you're using RStudio, you can see what's in your workspace on the top right.
## If not, use the ls() function to get a list of everything in the workspace.

ls()


## We see now that our workspace is empty, so let's add some stuff to it.
## First, let's add our 86, 14, 9 vector

## we use the "combine" or "concatenate" function to put elements into a vector.
example.vector <- c(86, 14, 9)

example.vector

## Now let's try the mean function again.
mean(example.vector)
(86 + 14 + 9) / 3 ## they give us the same answer! Woohooo!

## we can even store that mean as its own object
vec.mean <- mean(example.vector)
vec.mean

## Now let's store some other objects to our workspace
minutes <- 60 * 24 * 365 # numeric object that's the number of minutes in a year
minutes

## Objects don't have to be numeric, they can be character strings (letters/words)
name <- "Stephen"
name

mean(name) ## this won't work, since the "name" object isn't numeric


## We can reassign what's in an object, but it overwrites the original object
name
name <- c("Stephen", "Pettigrew")
name ## this object is called a "character vector,"
     ## since it's literally a vector (of length 2) of character elements




## Now let's check what's in our workspace again
ls()

## What is we want to get rid of the minutes object,
## how would we go about doing that?

??"remove object"

## rm() is the function we want
rm(minutes)
## The minutes object is gone
ls()

## what if we want to remove everything from our workspace?
rm(ls()) ## this doesn't work

rm(list = ls()) ## but this does














#######################################################
#### Other types of objects
#######################################################

######## VECTORS

### Three main types of data (although there's a bunch of others)
## Numeric
## Character
## Logical

########################################################################
## You've already seen numeric vectors
vector1 <- c(1,2,6,8,5)
vector1
mean(vector1)

###### Here's some other nifty ways to make numeric vectors:
## Using a colon between two numbers gives you all the integers between 

1:10

10:1

1:10.5


## You can use the rep() function to repeat the same value lots of times
# the synatax is: rep(thing to repeat, number of times to repeat it)
rep(x = 16, times = 3)
rep(16, 3)

#This also works with character vectors
rep("Lebron is the King", 5)




## You can use the seq() function to get patterns of numbers, like 3, 6, 9, 12
# syntax is: seq(where to start, where to end, what to count by)
seq(from = 3, to = 12, by = 3)
seq(3, 12, 3)

# They don't need to be integers
seq(2.4, 8.6, .15)



## We can even combine these functions together:
mixed.vector <- c(1:8, 
                  rep(16,4), 
                  rep("Lebron is the King", 4),
                  seq(3, 13, 3))
mixed.vector



example <- "mean(c(1,2,3,4))"


########################################################################
## You've also seen character vectors
vector2 <- c("thing1", 
             "thing2", 
             "string1",
             "string2",
             "red fish",
             "blue fish")
vector2
mean(vector2) ## doesn't work, since words don't have arthmetic means (obviously)






########################################################################
## Logical vectors are made up of trues and falses

## these two of code lines are identical:
vector3 <- c(T, F, T, T, F)
vector3 <- c(TRUE, FALSE, TRUE, TRUE, FALSE) 
vector3

## R treats logical vectors very similarly to numeric vectors
mean(vector3)





########################################################################
## Checking vector type and converting type

## How do we figure out what type of vector we have?

is.numeric(vector1) ##True
is.numeric(vector2) ##False
is.numeric(vector3) ##False

is.character(vector1) ##False
is.character(vector2) ##True
is.character(vector3) ##False

is.logical(vector1) ##False
is.logical(vector2) ##False
is.logical(vector3) ##True


## Like I said earlier, R treats logical vector similarly to numeric ones,
## but sometimes you want to make a logical vector into a numeric vector
vector3 <- as.numeric(vector3)
is.numeric(vector3) ##now this is true








########################################################################
## Can we put different types of objects into the same vector?

vector4 <- c("character", 2, T) 
vector4
## Yes we can, but if you have any character elements in the vector,
## it'll turn everything into characters


## In general, putting quotes around something will force it to be a character
vector5 <- c("8", "12", "16")
vector5

## this doesn't work because this is character vector, not numeric:
mean(vector5) 

## But if we convert to numeric, it does work
mean(as.numeric(vector5))


## Let's go back to our mixed.vector object
mixed.vector

## What if we want the means of the numeric values of the vector
## Let's start by converting it to numeric
mixed <- as.numeric(mixed.vector)

mean(mixed) ## it returns in NA!

## Using only the mean() function, write a line of code that gives you the 
## arthimetic mean of the mixed vector.
## Hint: look at the mean() mean help file by typing ?mean into the console
















##Answer:
mean(mixed, na.rm = T)









######################################################
## Other object classes
######################################################
## In addition to vectors, there are four major object classes in R:
## matrices, arrays, dataframes, and lists.
## Each of these is typically made up of numeric, character, and logical data


#############################################
## Matrix: Two dimensional (row x column) object
#############################################
## Hint: It's row by column (not column by row)
## because it's remote controlled (RC) cars, RC Cola,
## roller coaster, Roman Catholic, and Ray Charles

## Because if you mix things up and you make your matrix with
## dimension Charles Ray, you'll be doing statistics blindly


## let's put vector 2 into a 2x3 matrix

vector2

## The function we want to use is matrix(),
## which uses the syntax
matrix1 <- matrix(vector2, nrow = 2, ncol = 3)
matrix1

# it's a character matrix, since the vector we put in was made up of characters
is.character(matrix1)



## Let's put the mixed vector into a matrix
matrix2 <- matrix(mixed.vector, nrow = 4, ncol = 5)
# notice that everything is a character still
matrix2
# matrices can only have one type of data (character, numeric, logical)



## One thing to remember about matrices in R is that the data
## you put in them fills the matrix column by column, top to bottom
matrix(1:9, nrow = 3, ncol = 3)

## What if we want to fill the matrix to look like this:
## 1 2 3
## 4 5 6
## 7 8 9
# Write a line of code that'll do that














##Answer:
matrix(1:9, nrow = 3, ncol = 3, byrow = T)



## Write a line of code that fills up a 5 x 3 matrix with NA
## Hint: it's possible to do this in 14 keystrokes (not including spaces)












#Answer:
matrix(NA, nrow = 5, ncol = 3)
matrix(NA, 5, 3)



#############################################
###### Dataframe: a data frame is also a two-dimensional (row x column) object
#############################################

## Within each column the data must be of the same type
##(numeric, character, logical).
## Unlike matrices, though, different columns can have different
## data types.

## Regression and other statistical functions usually use dataframes
## Use as.data.frame()to convert matrices to dataframes

dataframe1 <- as.data.frame(matrix1)
dataframe2 <- as.data.frame(matrix2)

dataframe1
dataframe2






#############################################
## Arrays: Three dimensional (row x column x height) object
#############################################
## Working with arrays can be a pain in the butt.
## You've been warned

array1 <- array(0, dim = c(2, 2, 3))

array1


#############################################
## Lists: set of objects, each of which can be a different type
#############################################
# A single list could have 5 vectors, 4 matrices, 3 arrays,
# 2 dataframes, and a partidge in a peartree

ourlist <- list(vector1,
                vector2,
                dataframe1,
                dataframe2, 
                matrix1,
                array1)


ourlist









#######################################
### Other resources
#######################################

## When you can't figure out how to do something in R, google it!
## Often your google results will send you to a website called
## stackoverflow, which is a phenomenal resource for programming help
## in R (as well as Python, Stata, etc.)
